/*!
LTC2378_20: 20-Bit 1Msps ADC

@verbatim

The LTC2378_20 are pin-compatible, 20-bit A/D converters with 
serial I/O, and an internal reference. The ADCs typically draw only TBDmA from a 
single 5V supply. The automatic nap and sleep modes benefit power sensitive applications. 

The LTC2378_20's DC performance is outstanding with a +/-2LSB INL specification and 
no missing codes over temperature. The signal-to-noise ratio (SNR) for the LTC2378_20 is typically TBDdB, with the internal reference. 

@endverbatim

http://www.linear.com/product/LTC2378_20

http://www.linear.com/product/LTC2378_20#demoboards

REVISION HISTORY
$Revision: 1785 $
$Date: 2013-07-25 08:47:24 -0700 (Thu, 25 Jul 2013) $

Copyright (c) 2013, Linear Technology Corp.(LTC)
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

1. Redistributions of source code must retain the above copyright notice, this
   list of conditions and the following disclaimer.
2. Redistributions in binary form must reproduce the above copyright notice,
   this list of conditions and the following disclaimer in the documentation
   and/or other materials provided with the distribution.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

The views and conclusions contained in the software and documentation are those
of the authors and should not be interpreted as representing official policies,
either expressed or implied, of Linear Technology Corp.

The Linear Technology Linduino is not affiliated with the official Arduino team.
However, the Linduino is only possible because of the Arduino team's commitment
to the open-source community.  Please, visit http://www.arduino.cc and
http://store.arduino.cc , and consider a purchase that will help fund their
ongoing work.
*/

//! @defgroup LTC2378_20 LTC2378_20: 20-Bit 1Msps ADC

/*! @file
    @ingroup LTC2378_20
    Library for LTC2378_20: 20-Bit 1Msps ADC
*/

#include <Arduino.h>
#include <stdint.h>
#include "Linduino.h"
#include "LT_SPI.h"
#include "LTC2378_20.h"
#include <SPI.h>


// Reads the ADC  and returns 16-bit data
void LTC2378_20_read(uint8_t cs, uint16_t adc_command, uint32_t *ptr_adc_code)
{
//Note that the word 'adc_command' is a dummy variable, as there is no configuration of the ADC done over this interface.
//The contents of 'adc_command' are therefore "don't care".
//First break 'adc_command' into two bytes
//! This union splits one uint16_t (16-bit unsigned integer)
//! into two uint8_t's (8-bit unsigned integers) and vice versa.
  union two_byte_word
  {
    uint16_t word;  //!< 16-bit unsigned integer to be converted to two bytes
    uint8_t two_byte_array[2];  //!< 2 bytes (unsigned 8-bit integers) to be converted to a 16-bit signed or unsigned integer
  };

  two_byte_word temp_command_union;	//instantiate the union
  temp_command_union.word = adc_command;	//load 'adc_command' into the union

//Then form a four byte object incorporating these two bytes
//! This union splits one uint32_t (32-bit unsigned integer)
//! into four uint8_t's (8-bit unsigned integers) and vice versa.
  union four_byte_word
  {
    uint32_t word;  //!< 32-bit unsigned integer to be converted to four bytes
    uint8_t four_byte_array[4];  //!< 4 bytes (unsigned 8-bit integers) to be converted to a 32-bit signed or unsigned integer
  };

  four_byte_word command_union;	//instantiate the union
  command_union.four_byte_array[3] = temp_command_union.two_byte_array[1];	//load the contents of the 2 byte union into the 4 byte union
  command_union.four_byte_array[2] = temp_command_union.two_byte_array[0]; 
  command_union.four_byte_array[1] = 0; 
  command_union.four_byte_array[0] = 0; 

//Form a four byte object to hold four bytes of data
  four_byte_word data_union;	//instantiate the union
  data_union.four_byte_array[3] = 0; 
  data_union.four_byte_array[2] = 0; 
  data_union.four_byte_array[1] = 0; 
  data_union.four_byte_array[0] = 0; 

  spi_transfer_block(cs,command_union.four_byte_array,data_union.four_byte_array,3);

  *ptr_adc_code = data_union.word;
  *ptr_adc_code = *ptr_adc_code >>4;	//get rid of four meaningless LSBs

  return;
}


// Calculates the LTC2378_20 bipolar input voltage given the two's compliment data and lsb weight
float LTC2378_20_code_to_voltage(uint32_t adc_code, float LTC2378_20_lsb, int32_t LTC2378_20_offset_code)
{
  float adc_voltage, sign = 1.0;
  if ((adc_code>>19) == 1)
  {
    adc_code = (adc_code ^ 0xFFFFF)+1;                                    //! Convert ADC code from two's complement to binary
    sign = -1;
  }
  adc_voltage=((float)(adc_code+LTC2378_20_offset_code))*LTC2378_20_lsb*sign;    //! Calculate voltage from ADC code, lsb, offset.
  return(adc_voltage);
}


// Compute the offset in lsb given the two's compliment data
void LTC2378_20_cal_offset(uint32_t zero_code, uint32_t fs_code, float zero_voltage, float fs_voltage, float *LTC2378_20_lsb, int32_t *LTC2378_20_offset_code)
{
  float temp_offset;
  float zero_sign = 1.0;
  float fs_sign = 1.0;

  if ((zero_code>>19) == 1)
  {
    zero_code = (zero_code ^ 0xFFFFF)+1;                                    //! Convert ADC code from two's complement to binary
    zero_sign = -1;
  }

  if ((fs_code>>19) == 1)
  {
    fs_code = (fs_code ^ 0xFFFFF)+1;                                    //! Convert ADC code from two's complement to binary
    fs_sign = -1;
  }

 *LTC2378_20_lsb = (fs_voltage-zero_voltage)/((float)(fs_code*fs_sign - zero_code*zero_sign));          //!Calculate the LSB
  temp_offset = (zero_voltage / *LTC2378_20_lsb) - zero_code*zero_sign ;      


//!Calculate Bipolar offset
  temp_offset = (temp_offset > (floor(temp_offset) + 0.5)) ? ceil(temp_offset) : floor(temp_offset);    //!Round
  *LTC2378_20_offset_code = (int32_t)temp_offset; 
  return;                                                 //!cast as int32_t
}
